	/*
		Simple DirectX 11

		File:		Class_Direct3D.cpp
		Version:	0.1
		Author:		Nero

		Referenzen:
		http://www.directxtutorial.com/Tutorial11/tutorials.aspx
		http://www.rastertek.com/tutdx11.html	
		http://msdn.microsoft.com/en-us/library/windows/desktop/ee663274%28v=vs.85%29.aspx
	*/

	#include <Class_Direct3D.h>

	sdx_Direct3D::sdx_Direct3D()
	{
		this->sdx_D3D_Error					=	S_OK;
		this->sdx_D3D_Factory				=	NULL;
		this->sdx_D3D_Adapter				=	NULL;
		this->sdx_D3D_Output				=	NULL;
		this->sdx_D3D_SwapChain				=	NULL;
		this->sdx_D3D_Device				=	NULL;
		this->sdx_D3D_DeviceContext			=	NULL;	
		this->sdx_D3D_RenderTargetView		=	NULL;
		this->sdx_D3D_DepthStencilBuffer	=	NULL;
		this->sdx_D3D_DepthStencilState		=	NULL;
		this->sdx_D3D_DepthStencilView		=	NULL;
		this->sdx_D3D_RasterState			=	NULL;
		
		if(FAILED(CreateDXGIFactory1(__uuidof(IDXGIFactory1), (void**)(&this->sdx_D3D_Factory))))
		{
			this->sdx_D3D_Error = E_D3D_FACTORY;
		}
	}

	sdx_Direct3D::~sdx_Direct3D()
	{
		this->sdx_D3D_FreeScreen();
		if(this->sdx_D3D_Factory)				this->sdx_D3D_Factory->Release();	
		this->sdx_D3D_Factory				=	NULL;

	}

	void sdx_Direct3D::sdx_D3D_FreeScreen(void)
	{
		

		if(this->sdx_D3D_DeviceContext)			this->sdx_D3D_DeviceContext->Release();	
		if(this->sdx_D3D_Device)				this->sdx_D3D_Device->Release();
		if(this->sdx_D3D_SwapChain)
		{
			this->sdx_D3D_SwapChain->SetFullscreenState(FALSE, NULL);
			this->sdx_D3D_SwapChain->Release();
		}
		if(this->sdx_D3D_Output)				this->sdx_D3D_Output->Release();
		if(this->sdx_D3D_Adapter)				this->sdx_D3D_Adapter->Release();	
		if(this->sdx_D3D_RenderTargetView)		this->sdx_D3D_RenderTargetView->Release();
		if(this->sdx_D3D_DepthStencilBuffer)	this->sdx_D3D_DepthStencilBuffer->Release();
		if(this->sdx_D3D_DepthStencilState)		this->sdx_D3D_DepthStencilState->Release();
		if(this->sdx_D3D_DepthStencilView)		this->sdx_D3D_DepthStencilView->Release();
		if(this->sdx_D3D_RasterState)			this->sdx_D3D_RasterState->Release();
	
				
		this->sdx_D3D_Adapter				=	NULL;
		this->sdx_D3D_Output				=	NULL;
		this->sdx_D3D_SwapChain				=	NULL;
		this->sdx_D3D_Device				=	NULL;
		this->sdx_D3D_DeviceContext			=	NULL;	
		this->sdx_D3D_RenderTargetView		=	NULL;
		this->sdx_D3D_DepthStencilBuffer	=	NULL;
		this->sdx_D3D_DepthStencilState		=	NULL;
		this->sdx_D3D_DepthStencilView		=	NULL;
		this->sdx_D3D_RasterState			=	NULL;
		
		
	}

	HRESULT sdx_Direct3D::sdx_D3D_InitScreen(D3DAdapterSettings * Settings)
	{
		if(this->sdx_D3D_Adapter) return E_D3D_INIT_ADAPTER;


		if (!Settings->nearView) Settings->nearView = 1.0;
		if (!Settings->farView)  Settings->farView  = 100.0;

		this->sdx_D3D_VSync	= Settings->VSync;

		this->sdx_D3D_Error = this->sdx_D3D_Factory->EnumAdapters1(Settings->AdapterId, &this->sdx_D3D_Adapter);
		if(FAILED(this->sdx_D3D_Error)) return this->sdx_D3D_Error;

		this->sdx_D3D_Error = this->sdx_D3D_Adapter->EnumOutputs(Settings->OutputId, &this->sdx_D3D_Output);
		if(FAILED(this->sdx_D3D_Error)) return this->sdx_D3D_Error;

		DXGI_SWAP_CHAIN_DESC * SCDesc = (DXGI_SWAP_CHAIN_DESC*) PB_AllocateMemory(sizeof(DXGI_SWAP_CHAIN_DESC));
		
		this->sdx_D3D_FullScreen						= Settings->FullScreen;

		SCDesc->BufferCount								=	1;
		SCDesc->BufferDesc.Format						=	DXGI_FORMAT_R8G8B8A8_UNORM;
		SCDesc->BufferUsage								=	DXGI_USAGE_RENDER_TARGET_OUTPUT; 
		SCDesc->BufferDesc.ScanlineOrdering				=	DXGI_MODE_SCANLINE_ORDER_UNSPECIFIED;
		SCDesc->BufferDesc.Scaling						=	DXGI_MODE_SCALING_UNSPECIFIED;
		SCDesc->BufferDesc.Width						=	Settings->Width;
		SCDesc->BufferDesc.Height						=	Settings->Height;
		SCDesc->BufferDesc.RefreshRate.Numerator		=	0;
		SCDesc->BufferDesc.RefreshRate.Denominator		=	1;

		if (Settings->VSync)
		{
			SCDesc->BufferDesc.RefreshRate.Numerator	=	Settings->Numerator;
			SCDesc->BufferDesc.RefreshRate.Denominator	=	Settings->Denominator;
		}

		SCDesc->SampleDesc.Count						=	1;
		SCDesc->SampleDesc.Quality						=	0;

		if (Settings->MSAACount > 0)
		{
			SCDesc->SampleDesc.Count					=	Settings->MSAACount;
			SCDesc->SampleDesc.Quality					=	Settings->MSAAQuality;
		}

		SCDesc->OutputWindow							=	Settings->WindowId;
		SCDesc->Windowed								=	TRUE;
		
		SCDesc->SwapEffect								=	DXGI_SWAP_EFFECT_DISCARD;

		SCDesc->Flags									=	DXGI_SWAP_CHAIN_FLAG_ALLOW_MODE_SWITCH;

		D3D_FEATURE_LEVEL tmpFeatureLvl_Arr[]=
		{
			D3D_FEATURE_LEVEL_11_0,
			D3D_FEATURE_LEVEL_10_1,
			D3D_FEATURE_LEVEL_10_0
		};


		this->sdx_D3D_Error = D3D11CreateDeviceAndSwapChain(
											 this->sdx_D3D_Adapter, 
											 D3D_DRIVER_TYPE_UNKNOWN, 
											 NULL, 
											 D3D11_CREATE_DEVICE_BGRA_SUPPORT, 
											 tmpFeatureLvl_Arr, 
											 2, 
											 D3D11_SDK_VERSION, 
											 SCDesc, 
											 &this->sdx_D3D_SwapChain, 
											 &this->sdx_D3D_Device, 
											 &this->sdxFeatureLvl, 
											 &this->sdx_D3D_DeviceContext
											);
		PB_FreeMemory((DWORD*) SCDesc);


		if(FAILED(this->sdx_D3D_Error)) return this->sdx_D3D_Error;

		ID3D11Texture2D * tmpBackBuffer;
		this->sdx_D3D_Error = this->sdx_D3D_SwapChain->GetBuffer(0, __uuidof(ID3D11Texture2D), (LPVOID*) &tmpBackBuffer);
		if(FAILED(this->sdx_D3D_Error)) return this->sdx_D3D_Error;

		this->sdx_D3D_Error = this->sdx_D3D_Device->CreateRenderTargetView(tmpBackBuffer, NULL, &this->sdx_D3D_RenderTargetView);
		if(FAILED(this->sdx_D3D_Error)) return this->sdx_D3D_Error;
		
		tmpBackBuffer->Release();
		tmpBackBuffer = NULL;



		D3D11_TEXTURE2D_DESC * DBufferDesc = (D3D11_TEXTURE2D_DESC*) PB_AllocateMemory(sizeof(D3D11_TEXTURE2D_DESC));

		DBufferDesc->Width				= Settings->Width;
		DBufferDesc->Height				= Settings->Height;
		DBufferDesc->MipLevels			= 1;
		DBufferDesc->ArraySize			= 1;
		DBufferDesc->Format				= DXGI_FORMAT_D24_UNORM_S8_UINT;
		DBufferDesc->SampleDesc.Count	= 1;
		DBufferDesc->SampleDesc.Quality	= 0;
		DBufferDesc->Usage				= D3D11_USAGE_DEFAULT;
		DBufferDesc->BindFlags			= D3D11_BIND_DEPTH_STENCIL;
		DBufferDesc->CPUAccessFlags		= 0;
		DBufferDesc->MiscFlags			= 0;


		this->sdx_D3D_Error = this->sdx_D3D_Device->CreateTexture2D(DBufferDesc, NULL, &this->sdx_D3D_DepthStencilBuffer);
		if(FAILED(this->sdx_D3D_Error)) return this->sdx_D3D_Error;

		PB_FreeMemory((DWORD *) DBufferDesc);

		D3D11_DEPTH_STENCIL_DESC * DStencilDesc = (D3D11_DEPTH_STENCIL_DESC*) PB_AllocateMemory(sizeof(D3D11_DEPTH_STENCIL_DESC));
		
		DStencilDesc->DepthEnable					= true;
		DStencilDesc->DepthWriteMask				= D3D11_DEPTH_WRITE_MASK_ALL;
		DStencilDesc->DepthFunc						= D3D11_COMPARISON_LESS;
		DStencilDesc->StencilEnable					= true;
		DStencilDesc->StencilReadMask				= 0xFF;
		DStencilDesc->StencilWriteMask				= 0xFF;
		DStencilDesc->FrontFace.StencilFailOp		= D3D11_STENCIL_OP_KEEP;
		DStencilDesc->FrontFace.StencilDepthFailOp	= D3D11_STENCIL_OP_INCR;
		DStencilDesc->FrontFace.StencilPassOp		= D3D11_STENCIL_OP_KEEP;
		DStencilDesc->FrontFace.StencilFunc			= D3D11_COMPARISON_ALWAYS;
		DStencilDesc->BackFace.StencilFailOp		= D3D11_STENCIL_OP_KEEP;
		DStencilDesc->BackFace.StencilDepthFailOp	= D3D11_STENCIL_OP_DECR;
		DStencilDesc->BackFace.StencilPassOp		= D3D11_STENCIL_OP_KEEP;
		DStencilDesc->BackFace.StencilFunc			= D3D11_COMPARISON_ALWAYS;

		this->sdx_D3D_Error = this->sdx_D3D_Device->CreateDepthStencilState(DStencilDesc, &this->sdx_D3D_DepthStencilState);
		if(FAILED(this->sdx_D3D_Error)) return this->sdx_D3D_Error;

		this->sdx_D3D_DeviceContext->OMSetDepthStencilState(this->sdx_D3D_DepthStencilState, 1);
		
		PB_FreeMemory((DWORD *) DStencilDesc);

		D3D11_DEPTH_STENCIL_VIEW_DESC * DStencilViewDesc = (D3D11_DEPTH_STENCIL_VIEW_DESC*) PB_AllocateMemory(sizeof(D3D11_DEPTH_STENCIL_VIEW_DESC));

		DStencilViewDesc->Format					= DXGI_FORMAT_D24_UNORM_S8_UINT;
		DStencilViewDesc->ViewDimension				= D3D11_DSV_DIMENSION_TEXTURE2D;
		DStencilViewDesc->Texture2D.MipSlice		= 0;

		this->sdx_D3D_Error = this->sdx_D3D_Device->CreateDepthStencilView(this->sdx_D3D_DepthStencilBuffer, DStencilViewDesc, &this->sdx_D3D_DepthStencilView);
		if(FAILED(this->sdx_D3D_Error)) return this->sdx_D3D_Error;

		this->sdx_D3D_DeviceContext->OMSetRenderTargets(1, &this->sdx_D3D_RenderTargetView, this->sdx_D3D_DepthStencilView);

		PB_FreeMemory((DWORD *) DStencilViewDesc);

		D3D11_RASTERIZER_DESC * RasterDesc = (D3D11_RASTERIZER_DESC*) PB_AllocateMemory(sizeof(D3D11_RASTERIZER_DESC));

		RasterDesc->AntialiasedLineEnable	= false;
		RasterDesc->CullMode				= D3D11_CULL_BACK;
		RasterDesc->DepthBias				= 0;
		RasterDesc->DepthBiasClamp			= 0.0f;
		RasterDesc->DepthClipEnable			= true;
		RasterDesc->FillMode				= D3D11_FILL_SOLID;
		RasterDesc->FrontCounterClockwise	= false;
		RasterDesc->MultisampleEnable		= false;
		RasterDesc->ScissorEnable			= false;
		RasterDesc->SlopeScaledDepthBias	= 0.0f;
		
		this->sdx_D3D_Error = this->sdx_D3D_Device->CreateRasterizerState(RasterDesc, &this->sdx_D3D_RasterState);
		if(FAILED(this->sdx_D3D_Error)) return this->sdx_D3D_Error;

		PB_FreeMemory((DWORD *) RasterDesc);


		D3D11_VIEWPORT * VPort = (D3D11_VIEWPORT*) PB_AllocateMemory(sizeof(D3D11_VIEWPORT));

		VPort->Width		= (float)Settings->Width;
		VPort->Height		= (float)Settings->Height;
		VPort->MinDepth		= 0.0f;
		VPort->MaxDepth		= 1.0f;
		VPort->TopLeftX		= 0.0f;
		VPort->TopLeftY		= 0.0f;

		this->sdx_D3D_DeviceContext->RSSetViewports(1, VPort);
		PB_FreeMemory((DWORD *) VPort);

		float View		= (float) D3DX_PI / 4.0f;
		float Aspect	= (float) Settings->Width / (float) Settings->Height;

		D3DXMatrixPerspectiveFovLH(&this->sdx_D3D_ProjectionMatrix, View, Aspect, Settings->nearView, Settings->farView);
		D3DXMatrixIdentity(&this->sdx_D3D_WorldMatrix);
		D3DXMatrixOrthoLH(&this->sdx_D3D_OrthoMatrix, (float) Settings->Width, (float) Settings->Height, Settings->nearView, Settings->farView);
		

		if (this->sdx_D3D_FullScreen)
		{
			this->sdx_D3D_SwapChain->SetFullscreenState(this->sdx_D3D_FullScreen, NULL);
		}

		return S_OK;
	}



	void sdx_Direct3D::sdx_D3D_ClearScreen(DWORD Color)
	{
		if (!this->sdx_D3D_DeviceContext) return;
		this->sdx_D3D_DeviceContext->ClearRenderTargetView(this->sdx_D3D_RenderTargetView, D3DXCOLOR(Red(Color), Green(Color), Blue(Color), Alpha(Color)));
		this->sdx_D3D_DeviceContext->ClearDepthStencilView(this->sdx_D3D_DepthStencilView, D3D11_CLEAR_DEPTH, 1.0f, 0);		
	};

	void sdx_Direct3D::sdx_D3D_FlipBuffers()
	{
		if (!this->sdx_D3D_SwapChain) return;
		this->sdx_D3D_SwapChain->Present((UINT) this->sdx_D3D_VSync, 0);
	}
