	/*
		Simple DirectX 11

		File:		Class_ExamineAdapters.h
		Version:	0.2
		Author:		Nero

		Referenzen:
		http://www.spieleprogrammierer.de/27-tutorials/13263-direct3d11-settings-dialog-teil-1-enumerieren/
	*/

	#include <Class_ExamineAdapters.h>

	// sdx_E_Adapters Constructor
	sdx_E_Adapters::sdx_E_Adapters()
	{
		this->sdx_E_Error	= S_OK;
		this->sdx_E_Factory = NULL;

		if (FAILED(CreateDXGIFactory1(__uuidof(IDXGIFactory1), (void**)(&this->sdx_E_Factory))))
		{
			this->sdx_E_Error = E_EXA_FACTORY;
		}

	}

	// sdx_E_Adapters Destructor
	sdx_E_Adapters::~sdx_E_Adapters()
	{
		if(this->sdx_E_Factory) this->sdx_E_Factory->Release();

		this->sdx_E_ClearAdapterList();
		this->sdx_E_ClearOutputList();
		this->sdx_E_ClearVideoModeList();
		this->sdx_E_ClearMSAAList();

		this->sdx_E_Error	= NULL;
		this->sdx_E_Factory = NULL;
	}


	HRESULT sdx_E_Adapters::ExamineAdapters(DWORD * outPtr)
	{
		
		this->sdx_E_Error = this->sdx_E_GetAdapterList();
		if(FAILED(this->sdx_E_Error)) return this->sdx_E_Error;
		
		DWORD  myExaXML = PB_CreateXML2(PB_Any, 9);
		DWORD  mainNode = PB_CreateXMLNode(PB_RootXMLNode(myExaXML)); 
		PB_SetXMLNodeName(mainNode, L"AdapterList");
		
		//Adapter auflisten 
		for (DWORD x = 0; x < this->sdx_E_AdapterList.size(); x++)
		{
			DXGI_ADAPTER_DESC1 aDesc;
			this->sdx_E_Error = this->sdx_E_AdapterList.at(x)->GetDesc1(&aDesc);
			if(FAILED(this->sdx_E_Error)) return this->sdx_E_Error;

			DWORD  AdapterNode = PB_CreateXMLNode(mainNode);
			PB_SetXMLNodeName(AdapterNode, L"Adapter");		
			PB_SetXMLAttribute(AdapterNode, L"id", this->DWORDtoWChar(x));
			PB_SetXMLAttribute(AdapterNode, L"name", aDesc.Description);
			PB_SetXMLAttribute(AdapterNode, L"use", L"0");

			// Ausgaben auflisten
			if(FAILED(this->sdx_E_GetOutputList(x))) return E_OUTPUT_LIST;
			
			DWORD  OutputNodes = PB_CreateXMLNode(AdapterNode);
			PB_SetXMLNodeName(OutputNodes, L"OutputNodes");	

			for (DWORD o = 0; o < this->sdx_E_OutputList.size(); o++)
			{
				DXGI_OUTPUT_DESC oDesc;				
				this->sdx_E_Error = this->sdx_E_OutputList.at(x)->GetDesc(&oDesc);
				if(FAILED(this->sdx_E_Error)) return this->sdx_E_Error;

				DWORD OPNode = PB_CreateXMLNode(OutputNodes);
				PB_SetXMLNodeName(OPNode, L"Output");
				PB_SetXMLAttribute(OPNode, L"id", this->DWORDtoWChar(o));
				PB_SetXMLAttribute(OPNode, L"name",  oDesc.DeviceName);
				PB_SetXMLAttribute(OPNode, L"use",  L"0");

				// VideoModes auflisten
				if(FAILED(this->sdx_E_GetVideoModeList(o))) return E_VIDEOMODE_LIST;
				
				DWORD  VideoModesNodes = PB_CreateXMLNode(OPNode);
				PB_SetXMLNodeName(VideoModesNodes, L"VideoModes");	
			
				for (DWORD v = 0; v < this->sdx_E_VideoModeList.size(); v++)
				{
					DWORD ModeNode = PB_CreateXMLNode(VideoModesNodes);
					PB_SetXMLNodeName(ModeNode, L"Mode");

					PB_SetXMLAttribute(ModeNode, L"id",  this->DWORDtoWChar(v));
					PB_SetXMLAttribute(ModeNode, L"x",  this->DWORDtoWChar(this->sdx_E_VideoModeList.at(v).Width));
					PB_SetXMLAttribute(ModeNode, L"y",  this->DWORDtoWChar(this->sdx_E_VideoModeList.at(v).Height));
					PB_SetXMLAttribute(ModeNode, L"refresh",  this->DWORDtoWChar(this->sdx_E_VideoModeList.at(v).RefreshRate.Numerator / this->sdx_E_VideoModeList.at(v).RefreshRate.Denominator));
					PB_SetXMLAttribute(ModeNode, L"numerator",  this->DWORDtoWChar(this->sdx_E_VideoModeList.at(v).RefreshRate.Numerator));
					PB_SetXMLAttribute(ModeNode, L"denominator",  this->DWORDtoWChar(this->sdx_E_VideoModeList.at(v).RefreshRate.Denominator));
					PB_SetXMLAttribute(ModeNode, L"use",  L"0");
				}

				// Multisampling auflisten
				if(FAILED(this->sdx_E_GetMSAAList(o))) return E_MSAA_LIST;
				
				DWORD  MSAANodes = PB_CreateXMLNode(OPNode);
				PB_SetXMLNodeName(MSAANodes, L"Multisampling");
			
				for (DWORD m = 0; m < this->sdx_E_MSAAList.size(); m++)
				{
					DWORD ModeNode = PB_CreateXMLNode(MSAANodes);
					PB_SetXMLNodeName(ModeNode, L"Level");

					PB_SetXMLAttribute(ModeNode, L"id",  this->DWORDtoWChar(m));
					PB_SetXMLAttribute(ModeNode, L"count",  this->DWORDtoWChar(this->sdx_E_MSAAList.at(m).Count));
					PB_SetXMLAttribute(ModeNode, L"quality",  this->DWORDtoWChar(this->sdx_E_MSAAList.at(m).Qualities));
					PB_SetXMLAttribute(ModeNode, L"use",  L"0");
				}

			}


		}


		DWORD  AnisoNodes = PB_CreateXMLNode(mainNode);
		PB_SetXMLNodeName(AnisoNodes, L"Anisotropic");

		for(DWORD a = 2; a <= D3D11_MAX_MAXANISOTROPY; a *= 2) 
		{
			DWORD ALvlNode = PB_CreateXMLNode(AnisoNodes);
			PB_SetXMLNodeName(ALvlNode, L"Level");
			PB_SetXMLAttribute(ALvlNode, L"id",  this->DWORDtoWChar(a));
			PB_SetXMLAttribute(ALvlNode, L"use",  L"0");
		}

		DWORD  FSNodes = PB_CreateXMLNode(mainNode);
		PB_SetXMLNodeName(FSNodes, L"Fullscreen");
		PB_SetXMLAttribute(FSNodes, L"use",  L"0");

		DWORD  VSNodes = PB_CreateXMLNode(mainNode);
		PB_SetXMLNodeName(VSNodes, L"VSync");
		PB_SetXMLAttribute(VSNodes, L"use",  L"0");

		PB_FormatXML(myExaXML, 129);
		
		*outPtr = myExaXML;
		

		this->sdx_E_ClearAdapterList();
		this->sdx_E_ClearOutputList();
		this->sdx_E_ClearVideoModeList();
		this->sdx_E_ClearMSAAList();
		return S_OK;
	}