	/*
		Simple DirectX 11

		File:		Class_ExamineAdapters.h
		Version:	0.2
		Author:		Nero

		Referenzen:
		http://www.spieleprogrammierer.de/27-tutorials/13263-direct3d11-settings-dialog-teil-1-enumerieren/

	*/

	#include <Class_ExamineAdapters.h>

	// Kleine Funktion um DWORD in einen WCHAR array zu schreiben
	TCHAR * sdx_E_Adapters::DWORDtoWChar(DWORD Val)
	{		
		PB_FillMemory((DWORD*) this->tmpVal, 10);
		swprintf_s(this->tmpVal, L"%d", Val);
		return this->tmpVal;
	}

	// Alle Adapter enumerieren und in einen vector speichern
	HRESULT sdx_E_Adapters::sdx_E_GetAdapterList(void)
	{
		if(!this->sdx_E_Factory) return E_EXA_FACTORY;

		this->sdx_E_ClearAdapterList();			

		DWORD				AdapterCount = 0;
		IDXGIAdapter1	*	TmpAdapter;

		while(this->sdx_E_Factory->EnumAdapters1(AdapterCount++, &TmpAdapter) != DXGI_ERROR_NOT_FOUND) 
		{
			this->sdx_E_AdapterList.push_back(TmpAdapter);
			AdapterCount++;
		}

		if (this->sdx_E_AdapterList.empty()) return E_EXA_ADAPTERS;
		
		return S_OK;
	}

	// Alle Adaper freigeben und den vector leeren
	void sdx_E_Adapters::sdx_E_ClearAdapterList(void)
	{
		// Vectoren lschen & inhalte freigeben
		if (this->sdx_E_AdapterList.empty())
		{
			for (DWORD x = 0; x < this->sdx_E_AdapterList.size(); x++)  
			{
				if (this->sdx_E_AdapterList.at(x)) this->sdx_E_AdapterList.at(x)->Release();
			}
			this->sdx_E_AdapterList.clear();			
		}
		return;
	}

	

		// Alle mglichen ausgaben fr einen bestimmten Adapter in einen Vector speichern
	HRESULT sdx_E_Adapters::sdx_E_GetOutputList(DWORD AdapterID)
	{
		this->sdx_E_ClearOutputList();

		DWORD				OutputCount = 0;
		IDXGIOutput		*	TmpOutput;

		while(this->sdx_E_AdapterList.at(AdapterID)->EnumOutputs(OutputCount++, &TmpOutput) != DXGI_ERROR_NOT_FOUND)
		{
			this->sdx_E_OutputList.push_back(TmpOutput);
			OutputCount++;
		}
		if (this->sdx_E_OutputList.empty()) return E_FAIL;
		return S_OK;
	}

	// Alle Ausgaben freigeben und den vector leeren
	void sdx_E_Adapters::sdx_E_ClearOutputList(void)
	{
		// Vectoren lschen & inhalte freigeben
		if (this->sdx_E_OutputList.empty())
		{
			for (DWORD x = 0; x < this->sdx_E_OutputList.size(); x++)  this->sdx_E_OutputList.at(x)->Release();
			this->sdx_E_OutputList.clear();
		}
		return;
	}

	// Alle mglichen videomodes fr eine bestimmte Ausgabe in einen Vector speichern
	HRESULT sdx_E_Adapters::sdx_E_GetVideoModeList(DWORD OutputID)
	{
		this->sdx_E_ClearVideoModeList();

		DWORD ResolutionArrLen = 0;

		this->sdx_E_Error = this->sdx_E_OutputList.at(OutputID)->GetDisplayModeList(DXGI_FORMAT_R8G8B8A8_UNORM, 0, (UINT*) &ResolutionArrLen, NULL);
		if(FAILED(this->sdx_E_Error)) return this->sdx_E_Error;

		DXGI_MODE_DESC * TmpDescArr = new DXGI_MODE_DESC[ ResolutionArrLen];
		
		this->sdx_E_Error = this->sdx_E_OutputList.at(OutputID)->GetDisplayModeList(DXGI_FORMAT_R8G8B8A8_UNORM, 0, (UINT*) &ResolutionArrLen, TmpDescArr);
		if(FAILED(this->sdx_E_Error)) return this->sdx_E_Error;

		DWORD tmpX = 0, tmpY = 0, tmpR = 0, i = 0;
		
		for (DWORD x = 0; x < ResolutionArrLen; x++)
		{
			if ((tmpX != TmpDescArr[x].Width) && (tmpY != TmpDescArr[x].Height) && ((tmpX + tmpY) > 0))
			{
				this->sdx_E_VideoModeList.push_back(TmpDescArr[x]);
			}

			tmpX = TmpDescArr[x].Width;
			tmpY = TmpDescArr[x].Height;
		}

		delete[] TmpDescArr;

		if (this->sdx_E_VideoModeList.empty()) return E_FAIL;

		return S_OK;
	}

	// ViedeoModeList leeren
	void sdx_E_Adapters::sdx_E_ClearVideoModeList(void)
	{
		this->sdx_E_VideoModeList.clear();		
		return;
	}




		// MSAA und Qualitt in einen Vector speichern
	HRESULT sdx_E_Adapters::sdx_E_GetMSAAList(DWORD ID)
	{
		this->sdx_E_ClearMSAAList();

		D3D_FEATURE_LEVEL tmpFeatureLvl_Arr[]=
		{
			D3D_FEATURE_LEVEL_11_0,
			D3D_FEATURE_LEVEL_10_1,
			D3D_FEATURE_LEVEL_10_0
		};
		D3D_FEATURE_LEVEL		tmpFeatureLvl;
		ID3D11Device		*	tmpDevice;
		ID3D11DeviceContext *	tmpDeviceContext;
		
		tmpDeviceContext = NULL;
		tmpDevice = NULL;

		this->sdx_E_Error = D3D11CreateDevice(this->sdx_E_AdapterList.at(ID), D3D_DRIVER_TYPE_UNKNOWN, NULL, 0, tmpFeatureLvl_Arr, 2, D3D11_SDK_VERSION, &tmpDevice, &tmpFeatureLvl, &tmpDeviceContext);
		if(FAILED(this->sdx_E_Error)) return this->sdx_E_Error;

		for(DWORD x = 2; x < D3D11_MAX_MULTISAMPLE_SAMPLE_COUNT; x++)
		{

			DWORD msLVL = 0;
			if(FAILED(tmpDevice->CheckMultisampleQualityLevels(DXGI_FORMAT_R8G8B8A8_UNORM, x, (UINT*) &msLVL))) break;
			if(msLVL > 0) 
			{
				MSAA tmpMSAA;
				tmpMSAA.Count     = x;
				tmpMSAA.Qualities = msLVL - 1;

				this->sdx_E_MSAAList.push_back(tmpMSAA);				
			}
		}
		
		tmpDeviceContext->Release();
		tmpDevice->Release();


		if (this->sdx_E_MSAAList.empty()) return E_MSAA_LIST;

		return S_OK;
	}

	// MSAAList leeren
	void sdx_E_Adapters::sdx_E_ClearMSAAList(void)
	{
		this->sdx_E_MSAAList.clear();		
		return;
	}